/*
 * wc-cli.c
 * web conference client 
 * Copyright (c) 2003 Renzo Davoli <renzo@cs.unibo.it>
 * 
 * Some code has been imported from webspy
 * Copyright (c) 1999 Dug Song <dugsong@monkey.org>
 *
 */


#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <err.h>
#include <pwd.h>
#include <getopt.h>
#include <X11/Xlib.h>

#define SUCCESS 0
#define TRUE 1
#define FALSE !TRUE

#define SERV_TCP_PORT "9090"
static void fail(const char *message,int callexit);


/* for jwz's remote.c. */
extern int mozilla_remote_commands (Display *, Window, char **);
char	*expected_mozilla_version = "4.7";
char	*progname = "webconf-client";

static	Display		*dpy;
static	char	        cmd[2048], *cmdtab[2];
static	char		buf[2048];

static struct option long_options[]={
	{"ipv6", no_argument, 0, '6'},
	{"inet6", no_argument, 0, '6'},
	{"ipv4", no_argument, 0, '4'},
	{"inet", no_argument, 0, '4'},
	{"port", required_argument, 0, 'p'},
	{"display", required_argument, 0, 'h'},
	{"help", no_argument, 0, 'h'}
};

static void usage(const int e) {
	fprintf(stderr,"Usage: wc-cli [--ipv6 --inet6 --ipv4 --inet] [--port portno] server\n");
	fprintf(stderr,"Usage: wc-cli [-6 -4] [-p portno] server\n");
	exit(e);
}

int main(int argc, char *argv[])
{
	struct passwd *pw;
	int                     sockfd;
	struct addrinfo *res;
	struct addrinfo hints;
	char *service=SERV_TCP_PORT;
	int opt,option_index=0;
	char *display=getenv("DISPLAY");

	hints.ai_family=PF_UNSPEC;
	hints.ai_socktype=SOCK_STREAM;
	hints.ai_protocol=0;
	hints.ai_flags=AI_CANONNAME;

	while((opt=getopt_long_only(argc,argv,"hd:p:46",long_options,&option_index))>=0) {
		switch(opt) {
			case '4': hints.ai_family=PF_INET;
				break;
			case '6': hints.ai_family=PF_INET6;
				break;
			case 'p': service=strdup(optarg);
				break;
			case 'd': display=strdup(optarg);
				break;
			case 'h': usage(0);
				break;
			case '?': usage(-1);
				break;
		}
	}

	if (optind +1 != argc )
		usage(-1);

	//serverent=gethostbyname2(argv[1],AF_INET);
	if(getaddrinfo(argv[optind],service,&hints,&res)!=0)
		fail("getaddrinfo: Invalid Address",3);;

	if(res == NULL)
		fail("getaddrinfo returned NULL",3);
	if ((dpy = XOpenDisplay(display)) == NULL)
		fail("connection to local X server failed!",3);
	printf("connecting to %s\n",res->ai_canonname);

	if ( (sockfd = socket(res->ai_family, res->ai_socktype, 0)) < 0)
		fail("client: can't open stream socket",2);
	if (connect(sockfd, res->ai_addr,
				res->ai_addrlen) < 0)
		fail("client: can't connect to server",2);

	pw=getpwuid(geteuid());
	sprintf(buf,"  %s",pw->pw_name);
	buf[0]=4;
	buf[1]=2;
	write(sockfd,buf,strlen(buf)+1);
	sleep(10);
	while(read(sockfd,buf,sizeof(buf)) > 0)
	{
		cmdtab[0] = cmd;
		cmdtab[1] = NULL;
		printf("%s\n",buf);
		snprintf(cmd, sizeof(cmd), "openURL(%s)",
				buf);
		mozilla_remote_commands(dpy, 0, cmdtab);
	}
	close(sockfd);
	exit(0);
}


static void fail(const char *message,int callexit)
{
	fprintf(stderr,"ERROR: %s.\n",message);
	if (callexit!=FALSE)
		exit(callexit);
}
