/*
 * wc-server.c
 * web conference server 
 * Copyright (c) 2003 Renzo Davoli <renzo@cs.unibo.it>
 * 
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <sys/time.h>
#include <unistd.h>
#include <netdb.h>
#include <getopt.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define SUCCESS 0
#define TRUE 1
#define FALSE !TRUE
#define SERV_TCP_PORT "9090"
#define MAXBUF 4096

static void fail(const char *message,int callexit);

static int infifofd;
static char infifo[MAXBUF];
static char buf[MAXBUF];

static void wc_exit()
{
	exit(SUCCESS);
}

static void closeinfifo()
{
	close(infifofd);
	unlink(infifo);
}

void netunit(int newsockfd)
{
	register int size,reopen;
	fd_set selset;
	int fds;
	u_int ssize;
	
	struct sockaddr_storage sender;
	char numaddr[MAXBUF];

	ssize=sizeof(sender);
	memset(&sender, 0, sizeof(sender));
	getpeername(newsockfd,(struct sockaddr *)&sender,&ssize);
	if(getnameinfo((struct sockaddr *)&sender,ssize,numaddr,MAXBUF,NULL,0,NI_NUMERICHOST) < 0) {
		printf("sender id error\n");
		exit(0);
	}	
	size=read(newsockfd,buf,MAXBUF);
	if(size < 3 || buf[0] != 4 || buf[1] != 2)
	{
		printf("illegal request from %s\n",numaddr);
		exit(0);
	}
	sprintf(infifo,"fifo_%s_%s_%d",buf+2,numaddr,getpid());
	printf("%s has joined \n",infifo);
	unlink(infifo);
	mknod(infifo,0600|S_IFIFO,0);
	signal(SIGQUIT, closeinfifo);
	signal(SIGINT, closeinfifo);
	atexit(closeinfifo);

	if((fds=open("fifo_current",O_RDONLY)) >= 0) {
		size=read(fds,buf,MAXBUF);
		buf[size]=0;
		size=strcspn(buf,"\n");
		buf[size]=0;
		write(newsockfd,buf,size+1);
		close (fds);
	}

	//printf("fifo %d %s\n",getpidi),infifo);
	while (1) {
		//printf("open %d\n",getpid());
		infifofd=open(infifo, O_RDONLY);
		reopen=0;
		while(!reopen) {
			FD_ZERO(&selset);
			FD_SET(infifofd,&selset);
			FD_SET(newsockfd,&selset);
			select(16,&selset,NULL,NULL,NULL);
			if (FD_ISSET (newsockfd,&selset)) {
				size=read(newsockfd,buf,MAXBUF);
				//printf("return buf %d %s %d\n",getpid(),buf,size);
				if (size==0) {
					printf("%s has left\n",infifo);
					exit(0);
				}
			}
			if (FD_ISSET (infifofd,&selset)) {
				size=read(infifofd,buf,MAXBUF);
				if (size==0)
					reopen=1;
				else {
					//printf("buf %d %s %d\n",getpid(),buf,size);
					if(strncmp(buf,"exit:",5)==0) {
						printf("exit %s\n",infifo);
						exit(0);
					}
					buf[size]=0;
					write(newsockfd,buf,size+1);
				}
			}
		}
		//printf("close %d\n",getpid());
		close(infifofd);
	}
}

static int sockfd;

static void closesockfd()
{
	close(sockfd);
	unlink(infifo);
}

static struct option long_options[]={
	{"ipv6", no_argument, 0, '6'},
	{"inet6", no_argument, 0, '6'},
	{"ipv4", no_argument, 0, '4'},
	{"inet", no_argument, 0, '4'},
	{"port", required_argument, 0, 'p'},
	{"help", no_argument, 0, 'h'}
};

static void usage(const int e) {
	fprintf(stderr,"Usage: wc-server [--ipv6 --inet6 --ipv4 --inet] [--port portno] \n");
	fprintf(stderr,"Usage: wc-server [-6 -4] [-p portno] \n");
	exit(e);
}


int main(int argc, char **argv)
{
	int    newsockfd, childpid ;
	u_int  clilen;
	struct sockaddr cli_addr;

	struct addrinfo *res;
	struct addrinfo hints;
	char *service=SERV_TCP_PORT;
	int opt,option_index=0;

	hints.ai_family=PF_UNSPEC;
	hints.ai_socktype=SOCK_STREAM;
	hints.ai_protocol=0;
	hints.ai_flags=AI_PASSIVE;
	//hints.ai_flags=AI_CANONNAME|AI_PASSIVE;

	while((opt=getopt_long_only(argc,argv,"hd:p:46",long_options,&option_index))>=0) {
		switch(opt) {
			case '4': hints.ai_family=PF_INET;
				break;
			case '6': hints.ai_family=PF_INET6;
				break;
			case 'p': service=strdup(optarg);
				break;
			case 'h': usage(0);
				break;
			case '?': usage(-1);
				break;
		}
	}
		
	if (optind != argc )
		usage(-1);

	if(getaddrinfo(NULL,service,&hints,&res)!=0)
		fail("getaddrinfo: Invalid Address",3);;
	if(res == NULL)
		fail("getaddrinfo returned NULL",3);

	signal(SIGINT, wc_exit);
	signal(SIGQUIT, wc_exit);
	signal(SIGCHLD, SIG_IGN);
	if ( (sockfd = socket(res->ai_family, res->ai_socktype,  0)) < 0)
		                fail("server: can't open stream socket",2);
	atexit(closesockfd);
	//bzero((char *) &serv_addr, sizeof(serv_addr));
	//serv_addr.sin_family      = PF_INET;
	//serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
	//serv_addr.sin_port        = htons(SERV_TCP_PORT);

	if (bind(sockfd, res->ai_addr, res->ai_addrlen) < 0)
		                 fail("server: can't bind local address",2);

	listen(sockfd, 5);

	for ( ; ; ) {
		clilen = sizeof(cli_addr);
		newsockfd = accept(sockfd, (struct sockaddr *) &cli_addr, &clilen);
		//printf("new request\n");
		if (newsockfd < 0)
				fail("server: accept error",2);

		if ( (childpid = fork()) < 0)
		                fail("server: fork error",2);
		else if (childpid == 0) {       /* child process */
			close(sockfd);          /* close original socket */
			netunit(newsockfd);
			return(SUCCESS);
		}

		close(newsockfd);               /* parent process */

	}
}

static void fail(const char *message,int callexit)
{
	fprintf(stderr,"ERROR: %s.\n",message);
	if (callexit!=FALSE)
		exit(callexit);
}

